import os

from ..core.Simple import Simple
from ... import helper

FORM83 = "13"
FORM82 = "9"
FORM81 = "7"


class Form8x(Simple):
    ver = ''
    folder = "Формы"

    def __init__(self):
        super().__init__()
        self.form = None
        self.elements = None

    def decode_object(self, src_dir, uuid, dest_dir, dest_path, version, header):
        super(Form8x, self).decode_object(src_dir, uuid, dest_dir, dest_path, version, header)
        self.decode_data(src_dir, uuid)

    def write_decode_object(self, dest_dir, dest_path, version):
        super(Form8x, self).write_decode_object(dest_dir, dest_path, version)
        _dest_dir = os.path.join(dest_dir, dest_path)
        # helper.json_write(self.header, _dest_dir, f'{self.header["name"]}.json')
        helper.json_write(self.form, _dest_dir, f'{self.header["name"]}.form{self.ver}.json')
        # helper.txt_write(self.code, _dest_dir, f'{self.header["name"]}.1c')
        return []

    def decode_data(self, src_dir, uuid):
        raise NotImplemented()

    @classmethod
    def get_decode_header(cls, header):
        return cls.get_decode_obj_header(header)[1][1]

    @classmethod
    def get_decode_obj_header(cls, header):
        return header[0][1][1]

    def encode_object(self, src_dir, file_name, dest_dir, version):
        super(Form8x, self).encode_object(src_dir, file_name, dest_dir, version)
        try:
            self.form = helper.json_read(src_dir, f'{file_name}.form{version}.json')
        except FileNotFoundError:
            self.form = self.encode_empty_form()
        self.encode_data()

    def encode_header(self):
        raise NotImplemented()

    def encode_data(self):
        raise NotImplemented()

    def encode_empty_form(self):
        raise NotImplemented()

    def encode_write(self, dest_dir):
        raise NotImplemented()

    def write_encode_code(self, dest_dir):
        pass



from ..MetaDataObject import MetaDataObject
from ..MetaDataObject.versions.DocumentForm803 import DocumentForm803


class CatalogForm(MetaDataObject):
    versions = {
        '803': DocumentForm803
    }



import os

from .. import helper
from ...MetaDataObject import MetaDataObject
from ...ext_exception import ExtException


class Container(MetaDataObject):
    def __init__(self):
        super(Container, self).__init__()
        self.new_dest_path = None
        self.new_dest_dir = None

    @classmethod
    def get_decode_header(cls, header):
        return header[0][1][1]

    @classmethod
    def get_decode_includes(cls, header_data):
        try:
            return [header_data[0]]
        except IndexError:
            raise ExtException(msg='Include types not found', detail=cls.__name__)

    def decode_object(self, src_dir, file_name, dest_dir, dest_path, version, header_data):
        super(Container, self).decode_object(src_dir, file_name, dest_dir, dest_path, version, header_data)
        self.decode_code(src_dir)
        self.new_dest_path = os.path.join(dest_path, self.header['name'])
        self.new_dest_dir = os.path.join(dest_dir, self.new_dest_path)
        os.makedirs(self.new_dest_dir)

    def write_decode_object(self, dest_dir, dest_path, version):
        helper.json_write(self.header, self.new_dest_dir, f'{self.__class__.__name__}.json')
        self.write_decode_code(self.new_dest_dir, self.__class__.__name__)

    def decode_includes(self, src_dir, dest_dir, dest_path, header_data):
        return super(Container, self).decode_includes(src_dir, dest_dir, self.new_dest_path, header_data)

    def encode_object(self, src_dir, file_name, dest_dir, version):
        self.encode_code(src_dir, self.__class__.__name__)
        return []

    @classmethod
    def encode_get_include_obj(cls, src_dir, dest_dir, include, tasks, version):
        """
        возвращает список задач на парсинг объектов этого типа
        """
        entries = os.listdir(src_dir)
        for entry in entries:
            if os.path.isdir(os.path.join(src_dir, entry)):
                new_src_dir = os.path.join(src_dir, entry)
                tasks.append([include, [new_src_dir, include, dest_dir, version]])



from ...MetaDataObject import MetaDataObject


class Simple(MetaDataObject):
    def __init__(self):
        super(Simple, self).__init__()

    def decode_object(self, src_dir, file_name, dest_dir, dest_path, version, header_data):
        super(Simple, self).decode_object(src_dir, file_name, dest_dir, dest_path, version, header_data)
        self.decode_code(src_dir)

    def write_decode_object(self, dest_dir, dest_path, version):
        super(Simple, self).write_decode_object(dest_dir, dest_path, version)

    def decode_includes(self, src_dir, dest_dir, dest_path, header):
        return []

    def encode_includes(self, src_dir, dest_dir):
        return []

    def encode_object(self, src_dir, file_name, dest_dir, version):
        self.encode_code(src_dir, self.header["name"])
        return []
